Package: doBy
Version: 4.7.1
Title: Groupwise Statistics, LSmeans, Linear Estimates, Utilities
Authors@R: c(
    person(given = "Ulrich", family = "Halekoh",
        email = "uhalekoh@health.sdu.dk", role = c("aut", "cph")),
    person(given = "Søren", family = "Højsgaard",
        email = "sorenh@math.aau.dk", role = c("aut", "cre", "cph"))
  )
Description: Utility package containing: Main categories: Working with grouped data: 'do' something to data when  stratified 'by' some variables.  General linear estimates. Data handling utilities. Functional programming, in particular restrict functions to a smaller domain. Miscellaneous functions for data handling. Model stability in connection with model selection. Miscellaneous other tools. 
Encoding: UTF-8
VignetteBuilder: knitr
LazyData: true
LazyDataCompression: xz
URL: https://github.com/hojsgaard/doBy
License: GPL (>= 2)
Depends: R (>= 4.2.0), methods
Imports: boot, broom, cowplot, Deriv, dplyr, forecast, ggplot2, MASS,
        Matrix, modelr, microbenchmark, rlang, purrr, tibble, tidyr,
Suggests: geepack, knitr, lme4, markdown, rmarkdown, multcomp, pbkrtest
        (>= 0.5.2), survival, testthat (>= 2.1.0)
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-01 22:47:07 UTC; sorenh
Author: Ulrich Halekoh [aut, cph],
  Søren Højsgaard [aut, cre, cph]
Maintainer: Søren Højsgaard <sorenh@math.aau.dk>
Repository: CRAN
Date/Publication: 2025-12-02 06:10:39 UTC
Built: R 4.6.0; ; 2026-04-25 14:00:36 UTC; unix
