## doBy

A utility package: Groupwise Statistics, LSmeans, Linear Contrasts, Utilities

1. Facilities for working with grouped data: 'do' something to data stratified 'by' some variables.
2. Linear estimates and LSmeans (marginal means)
3. Functional  programming
4. Pipe arithmetic
5. Flexible variable selection
6. Miscellaneous datasets

Author: Søren Højsgaard,  Ulrich Halekoh

### Online documentation

https://hojsgaard.github.io/doBy

### Development site

https://github.com/hojsgaard/doBy

### CRAN page:

https://cran.r-project.org/web/packages/doBy/index.html


