% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vparse}
\alias{vparse}
\title{Variable utilities: parse, select, check, map, rename}
\usage{
vparse(...)
}
\arguments{
\item{...}{Variable input (unquoted, character vector, or formula)}
}
\description{
These functions provide flexible tools for parsing and managing variable names
from formulas, unquoted names, or character vectors. Demonstrated using \code{CO2} dataset.
}
