% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linear_modelling.R
\name{reciprocal}
\alias{reciprocal}
\title{Reciprocal function}
\usage{
reciprocal(x)
}
\arguments{
\item{x}{An R object for whih 1/x makes sense}
}
\description{
A simple function returning the reciprocal of its argument
}
\author{
Søren Højsgaard
}
