% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_mathmark}
\alias{data_mathmark}
\alias{mathmark}
\alias{math}
\title{Mathematics marks for students}
\format{
This data frame contains the following columns: mechanics, vectors,
algebra, analysis, statistics.
}
\usage{
data(mathmark)
}
\description{
The \code{mathmark} data frame has 88 rows and 5 columns.
}
\examples{

data(mathmark)

}
\references{
David Edwards, An Introduction to Graphical Modelling, Second
Edition, Springer Verlag, 2000
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{dataset}
\keyword{datasets}
