% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{crime_rate}
\alias{crime_rate}
\title{crimeRate}
\format{
This data frame contains:
\describe{
\item{murder:}{crime of murder}
\item{rape:}{}
\item{robbery:}{}
\item{assault:}{}
\item{burglary:}{residential theft}
\item{larceny:}{unlawful taking of personal property (pocket picking)}
\item{autotheft:}{}
}
}
\usage{
crime_rate
}
\description{
Crime rates per 100,000 inhabitants in states of the USA for different crime
types in 1977.
}
\examples{
	
data(crime_rate)

}
\concept{dataset}
\keyword{datasets}
